@@ -5,11 +5,10 @@ package Data::Scan;
 
 # ABSTRACT: Stackfree arbitrary data scanner
 
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
-# AUTHORITY
+our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
-use Carp qw/croak/;
 use Moo;
 use Scalar::Util qw/refaddr reftype/;
 use Types::Standard qw/ConsumerOf/;
@@ -27,6 +26,11 @@ my $openaddr = \$_open;
 my $closeaddr = \$_close;
 
 
+#
+# Avoid calls to arybase
+#
+my $ARRAY_START_INDICE = $[;
+
 sub process {
   my ($self) = shift;
 
@@ -42,10 +46,10 @@ sub process {
     #
     # First our private thingies
     #
-    while (@_ && ref $_[$[]) {
-      if    ($openaddr  == refaddr $_[$[]) { $consumer->dsopen ((splice @_, $[, 2)[-1]) }
-      elsif ($closeaddr == refaddr $_[$[]) { $consumer->dsclose((splice @_, $[, 2)[-1]) }
-      else                                 { last }
+    while (@_ && ref $_[$ARRAY_START_INDICE]) {
+      if    ($openaddr  == refaddr $_[$ARRAY_START_INDICE]) { $consumer->dsopen ((splice @_, $ARRAY_START_INDICE, 2)[-1]) }
+      elsif ($closeaddr == refaddr $_[$ARRAY_START_INDICE]) { $consumer->dsclose((splice @_, $ARRAY_START_INDICE, 2)[-1]) }
+      else                                                  { last }
     }
     #
     # Consumer's dsread() returns eventual inner content
@@ -77,7 +81,7 @@ Data::Scan - Stackfree arbitrary data scanner
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -98,7 +102,7 @@ Data::Scan is a stackfree scanner of arbitrary data. It has no other intelligenc
 
 =head2 consumer
 
-A object instance that is consuming the Data::Scan::Role::Consumer role.
+A object instance that is consuming the Data::Scan::Role::Consumer role. L<Data::Scan::Printer> is an example of an implementation consuming this role.
 
 =head1 SUBROUTINES/METHODS
 
@@ -138,7 +142,7 @@ Indicates to the consumer that scanning is ending. Return value of consumer->end
 
 =head1 SEE ALSO
 
-L<Data::Scan::Role::Consumer>
+L<Data::Scan::Role::Consumer>, L<Data::Scan::Printer>
 
 =for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
