@@ -5,14 +5,15 @@ package Data::Scan::Impl::Printer;
 
 # ABSTRACT: Data::Scan printer implementation
 
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
-# AUTHORITY
+our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
 
+use Moo;
+
 use B::Deparse;
 use Class::Inspector;
-use Moo;
 use Perl::OSType qw/is_os_type/;
 my $_HAVE_Win32__Console__ANSI;
 BEGIN {
@@ -28,8 +29,16 @@ use Types::Common::Numeric -all;
 #
 # My way of matching only printable ASCII characters
 #
-my $_ASCII_PRINT = quotemeta(join('', map { chr } 33..126));
+my $_ASCII_PRINT = quotemeta(join('', map { chr } (32,33..126)));
 my $_NON_ASCII_PRINT_RE = qr/[^$_ASCII_PRINT]/;
+#
+# Avoid calls to arybase and predictible results
+#
+my $ARRAY_START_INDICE         = $[;
+my $ARRAY_START_INDICE_PLUS_1  = $ARRAY_START_INDICE + 1;
+my $ARRAY_START_INDICE_PLUS_2  = $ARRAY_START_INDICE_PLUS_1 + 1;
+my $ARRAY_START_INDICE_PLUS_3  = $ARRAY_START_INDICE_PLUS_2 + 1;
+my $ARRAY_START_INDICE_MINUS_1 = $ARRAY_START_INDICE - 1;
 
 
 
@@ -75,13 +84,13 @@ has hash_end          => (is => 'ro', isa => Str,              default => sub {
 has hash_separator    => (is => 'ro', isa => Str,              default => sub { return ' => '    });
 
 
-has indice_start      => (is => 'ro', isa => Str,              default => sub { return '['       });
+has indice_start          => (is => 'ro', isa => Str,              default => sub { return '['       });
 
 
-has indice_end        => (is => 'ro', isa => Str,              default => sub { return '] '      });
+has indice_end          => (is => 'ro', isa => Str,              default => sub { return '] '      });
 
 
-has with_indices_full => (is => 'ro', isa => Bool,             default => sub { return !!1       });
+has with_indices_full => (is => 'ro', isa => Bool,             default => sub { return !!0       });
 
 
 has address_start     => (is => 'ro', isa => Str,              default => sub { return '('       });
@@ -111,13 +120,13 @@ has with_hash_indice  => (is => 'ro', isa => Bool,             default => sub {
 has with_deparse      => (is => 'ro', isa => Bool,             default => sub { return !!0       });
 
 
-has with_methods      => (is => 'ro', isa => Bool,             default => sub { return !!1       });
+has with_methods      => (is => 'ro', isa => Bool,             default => sub { return !!0       });
 
 
 has with_filename     => (is => 'ro', isa => Bool,             default => sub { return !!0       });
 
 
-has colors            => (is => 'ro', isa => HashRef[Str|ArrayRef|Undef], default => sub {
+has colors            => (is => 'ro', isa => HashRef[Str|Undef], default => sub {
                             return {
                                     blessed         => 'bold',
                                     string          => undef,
@@ -148,15 +157,24 @@ has colors            => (is => 'ro', isa => HashRef[Str|ArrayRef|Undef], defaul
                                     code            => 'yellow',
                                     already_scanned => 'green'
                                    }
-                          });
+                          }
+                         );
 #
-# Internal attributes
+# Internal attributes - Note that they are ALL explicitely setted in dsstart()
+# and because they are internal we give us the right to access them
+# in the dirty way
 #
-has _lines                  => (is => 'rw', isa => ArrayRef,                    clearer => 1, lazy => 1, default => sub { [ '' ] });
-has _currentLevel           => (is => 'rw', isa => PositiveOrZeroInt,           clearer => 1, lazy => 1, default => sub { 0 });
-has _currentIndicePerLevel  => (is => 'rw', isa => ArrayRef[PositiveOrZeroInt], clearer => 1, lazy => 1, default => sub { [] });
-has _currentReftypePerLevel => (is => 'rw', isa => ArrayRef[Str],               clearer => 1, lazy => 1, default => sub { [] });
-has _seen                   => (is => 'rw', isa => HashRef[PositiveOrZeroInt],  clearer => 1, lazy => 1, default => sub { {} });
+has _lines                  => (is => 'rwp', isa => Undef|ArrayRef[ArrayRef]);
+has _currentLevel           => (is => 'rwp', isa => Undef|PositiveOrZeroInt);
+has _currentIndicePerLevel  => (is => 'rwp', isa => Undef|ArrayRef[PositiveOrZeroInt]);
+has _currentReftypePerLevel => (is => 'rwp', isa => Undef|ArrayRef[Str]);
+has _seen                   => (is => 'rwp', isa => Undef|HashRef[PositiveOrZeroInt]);
+has _indice_start_nospace   => (is => 'rwp', isa => Undef|Str);  # C.f. BUILD
+has _indice_end_nospace     => (is => 'rwp', isa => Undef|Str);
+has _colors_cache           => (is => 'rwp', isa => Undef|HashRef[Str|Undef]);
+has _concatenatedLevels     => (is => 'rwp', isa => Undef|ArrayRef[Str]);
+has _output                 => (is => 'rwp', isa => Str);
+
 #
 # Required methods
 #
@@ -164,27 +182,82 @@ has _seen                   => (is => 'rw', isa => HashRef[PositiveOrZeroInt],
 
 sub dsstart  {
   my ($self) = @_;
-  $self->_clear_lines;
-  $self->_clear_currentLevel;
-  $self->_clear_currentIndicePerLevel;
-  $self->_clear_currentReftypePerLevel;
-  $self->_clear_seen;
+
+  $self->_set__lines([[]]);
+  $self->_set__currentLevel(0);
+  $self->_set__currentIndicePerLevel([]);
+  $self->_set__currentReftypePerLevel([]);
+  $self->_set__seen({});
+  $self->_set__concatenatedLevels([]);
+
+  my $indice_start_nospace = $self->indice_start;
+  my $indice_end_nospace = $self->indice_end;
+  $indice_start_nospace =~ s/\s//g;
+  $indice_end_nospace =~ s/\s//g;
+  $self->_set__indice_start_nospace($indice_start_nospace);
+  $self->_set__indice_end_nospace($indice_end_nospace);
+  #
+  # Precompute color attributes
+  #
+  $self->_set__colors_cache({});
+  if ($self->with_ansicolor) {
+    foreach (keys %{$self->colors}) {
+      my $color = $self->colors->{$_};
+      if (defined($color)) {
+        my $colored = colored('dummy', $color);
+        #
+        # ANSI color spec is clear: attributes before the string, followed by
+        # the string, followed by "\e[0m". We do not support the eventual
+        # $EACHLINE hack.
+        #
+        if ($colored =~ /(.+)dummy\e\[0m$/) {
+          $self->{_colors_cache}->{$_} = substr($colored, $-[1], $+[1] - $-[1])
+        } else {
+          $self->{_colors_cache}->{$_} = undef
+        }
+      } else {
+        $self->{_colors_cache}->{$_} = undef
+      }
+    }
+  } else {
+    foreach (keys %{$self->colors}) {
+      $self->{_colors_cache}->{$_} = undef
+    }
+  }
+
+  $self->_set__output('');
+
   return
 }
 
 
-sub dsend { return !!1 }
+sub dsend {
+  my ($self) = @_;
+
+  $self->_set__output(join($self->newline, map { join('', @{$_}) } @{$self->{_lines}}));
+
+  $self->_set__lines                 (undef);
+  $self->_set__currentLevel          (undef);
+  $self->_set__currentIndicePerLevel (undef);
+  $self->_set__currentReftypePerLevel(undef);
+  $self->_set__seen                  (undef);
+  $self->_set__indice_start_nospace  (undef);
+  $self->_set__indice_end_nospace    (undef);
+  $self->_set__colors_cache          (undef);
+  $self->_set__concatenatedLevels    (undef);
+
+  return !!1
+}
 
 
 sub dsprint {
   my ($self, $handle) = @_;
 
   $handle //= $self->handle;
-  my $string = join($self->newline, @{$self->_lines});
   if (Scalar::Util::blessed($handle) && $handle->can('print')) {
-    return $handle->print($string)
+    return $handle->print($self->_output)
   } else {
-    return print $handle $string
+    return print $handle $self->_output
   }
 }
 
@@ -199,7 +272,22 @@ sub dsopen {
   elsif ($reftype eq 'HASH')  { $self->_pushDesc('hash_start',  $self->hash_start)  }
   else                        { $self->_pushDesc('ref_start',   $self->ref_start)   }
 
+  #
+  # Precompute the string describing previous level.
+  # Here $self->{_currentLevel} is the value before we increase it
+  #
+  if ($self->{_currentLevel}) {
+    push(@{$self->{_concatenatedLevels}},
+         $self->{_concatenatedLevels}->[-1] .
+         $self->_indice_start_nospace .
+         $self->{_currentIndicePerLevel}->[-1] .
+         $self->_indice_end_nospace)
+  } else {
+    push(@{$self->{_concatenatedLevels}}, '')
+  }
+
   $self->_pushLevel($reftype);
+
   return
 }
 
@@ -207,6 +295,11 @@ sub dsopen {
 sub dsclose {
   my ($self, $item) = @_;
 
+  #
+  # Remove precomputed string describing this level.
+  #
+  pop(@{$self->{_concatenatedLevels}});
+
   $self->_popLevel;
 
   my $reftype = reftype $item;
@@ -224,58 +317,61 @@ sub dsread {
   my $refaddr = refaddr($item);
   my $blessed = Scalar::Util::blessed($item) // '';
   my $reftype = reftype($item) // '';
-
+  #
+  # Precompute things that always have the same value
+  #
+  my $indice_start                   = $self->indice_start;
+  my $indice_end                     = $self->indice_end;
+  my $indice_start_nospace           = $self->_indice_start_nospace;
+  my $indice_end_nospace             = $self->_indice_end_nospace;
+  #
+  # Increase indice if we reading something unfolded
+  #
+  my $currentLevel = $self->{_currentLevel};
+  my $currentIndicePerLevel = $currentLevel ? ++$self->{_currentIndicePerLevel}->[-1] : undef;
   #
   # Push a newline or a '=>' and prefix with indice if in a fold
   #
-  my $currentLevel = $self->_currentLevel;
   if ($currentLevel) {
-    my $currentReftypePerLevel = $self->_currentReftypePerLevel->[-1];
-    my $currentIndicePerLevel = $self->_currentIndicePerLevel->[-1];
+    my $currentReftypePerLevel = $self->{_currentReftypePerLevel}->[-1];
     if ($currentReftypePerLevel eq 'ARRAY' or $currentReftypePerLevel eq 'HASH') {
       my $show_indice;
       if ($currentReftypePerLevel eq 'ARRAY') {
-        $self->_pushDesc('array_next', $self->array_next) if ($currentIndicePerLevel > $[);
+        $self->_pushDesc('array_next', $self->array_next) if ($currentIndicePerLevel > $ARRAY_START_INDICE);
         $self->_pushLine;
         $show_indice = $self->with_array_indice
       } else {
         if ($currentIndicePerLevel % 2) {
-          $self->_pushDesc('hash_separator', $self->hash_separator);
+          $self->_pushDesc('hash_separator', $self->hash_separator)
         } else {
           $self->_pushDesc('hash_next', $self->hash_next) if ($currentIndicePerLevel > 0);
-          $self->_pushLine;
+          $self->_pushLine
         }
         $show_indice = $self->with_hash_indice
       }
       if ($show_indice) {
         if ($self->with_indices_full) {
-          my @levels = (
-                        $self->_concatenateLevels($currentLevel - 1),
-                        $self->indice_start . $currentIndicePerLevel . $self->indice_end
-                       );
           #
-          # As in a \var reference (see below), we want the levels to not have eventual space
+          # We know that $self->{_concatenatedLevels} is an ArrayRef.
+          # $currentLevel is a true value, this mean there is at least
+          # one element in $self->{_concatenatedLevels}.
           #
-          my $levels = join('', @levels);
-          $levels =~ s/\s//g;
-          $self->_pushDesc('indice_full', $levels)
+          $self->_pushDesc('indice_full', $self->{_concatenatedLevels}->[-1] . $indice_start_nospace . $currentIndicePerLevel . $indice_end_nospace)
         } else {
-          $self->_pushDesc('indice_start', $self->indice_start);
+          $self->_pushDesc('indice_start', $indice_start);
           $self->_pushDesc('indice_value', $currentIndicePerLevel);
-          $self->_pushDesc('indice_end', $self->indice_end)
+          $self->_pushDesc('indice_end', $indice_end)
         }
       }
     }
-    $self->_currentIndicePerLevel->[-1]++
   }
   #
   # See how this can be displayed
   #
   my $alreadyScanned;
   if ($refaddr) {
-    my $seen = $self->_seen;
-    if (exists $seen->{$refaddr}) {
-      $alreadyScanned = $seen->{$refaddr};
+    if (exists($self->{_seen}->{$refaddr})) {
+      $alreadyScanned = $self->{_seen}->{$refaddr};
       #
       # Already scanned !
       #
@@ -283,14 +379,13 @@ sub dsread {
     } else {
       #
       # Determine the "location" in terms of an hypothetical "@var" describing the tree
-      # Note that we already increased $currentLevel
-      my @levels = $self->_concatenateLevels($currentLevel);
       #
-      # We want the levels to not have eventual space
+      my $var = 'var';
       #
-      my $levels = join('', @levels);
-      $levels =~ s/\s//g;
-      $seen->{$refaddr} = "var$levels"
+      # Note the if ($currentLevel) at the end
+      #
+      $var .= $self->{_concatenatedLevels}->[-1] . $indice_start_nospace . $currentIndicePerLevel . $indice_end_nospace if ($currentLevel);
+      $self->{_seen}->{$refaddr} = $var
     }
   }
   if (! $alreadyScanned) {
@@ -304,21 +399,21 @@ sub dsread {
       #
       # Code deparse with B::Deparse
       #
-      my $i = length($self->indent) x ($self->_currentLevel + 2);
+      my $i = length($self->indent) x ($self->{_currentLevel} + 2);
       my $deparseopts = ["-sCv'Useless const omitted'"];
       my $code = 'sub ' . B::Deparse->new($deparseopts)->coderef2text($item);
       my @code = split(/\R/, $code);
       #
-      # First item is no aligned
+      # First item is not aligned
       #
       $self->_pushDesc('code', shift(@code));
       #
-      # The first is aligned
+      # The rest is aligned
       #
       if (@code) {
         $self->_pushLevel($reftype);
         map { $self->_pushLine; $self->_pushDesc('code', $_) } @code;
-        $self->_popLevel($reftype)
+        $self->_popLevel
       }
     } elsif ((! $reftype)
                ||
@@ -349,8 +444,9 @@ sub dsread {
     # Show address ?
     #
     if ($refaddr && $self->with_address) {
+      my $address_format = $self->address_format;
       $self->_pushDesc('address_start', $self->address_start);
-      $self->_pushDesc('address_value', length($self->address_format) ? sprintf($self->address_format, $refaddr) : $refaddr);
+      $self->_pushDesc('address_value', length($address_format) ? sprintf($address_format, $refaddr) : $refaddr);
       $self->_pushDesc('address_end', $self->address_end)
     }
   }
@@ -386,12 +482,19 @@ sub dsread {
       my $expanded = Class::Inspector->methods($blessed, 'expanded');
       if (defined($expanded) && reftype($expanded) eq 'ARRAY') {
         my @expanded = @{$expanded};
-        my $class_idx = $[ + 1;
-        my $method_idx = $class_idx + 1;
-        my $ref_idx = $method_idx + 1;
-        my %public_methods    = map { $_->[$method_idx] => $_->[$ref_idx] } grep { $_->[$method_idx] !~ /^\_/   } grep { $_->[$class_idx] eq $blessed } @expanded;
-        my %private_methods   = map { $_->[$method_idx] => $_->[$ref_idx] } grep { $_->[$method_idx] =~ /^\_/   } grep { $_->[$class_idx] eq $blessed } @expanded;
-        my %inherited_methods = map { $_->[$method_idx] => $_->[$ref_idx] }                                       grep { $_->[$class_idx] ne $blessed } @expanded;
+        my %public_methods =
+          map  { $_->[$ARRAY_START_INDICE_PLUS_2] => $_->[$ARRAY_START_INDICE_PLUS_3] }
+          grep { $_->[$ARRAY_START_INDICE_PLUS_2] !~ /^\_/   }
+          grep { $_->[$ARRAY_START_INDICE_PLUS_1] eq $blessed }
+          @expanded;
+        my %private_methods =
+          map  { $_->[$ARRAY_START_INDICE_PLUS_2] => $_->[$ARRAY_START_INDICE_PLUS_3] }
+          grep { $_->[$ARRAY_START_INDICE_PLUS_2] =~ /^\_/   }
+          grep { $_->[$ARRAY_START_INDICE_PLUS_1] eq $blessed }
+          @expanded;
+        my %inherited_methods =
+          map  { $_->[$ARRAY_START_INDICE_PLUS_2] => $_->[$ARRAY_START_INDICE_PLUS_3] }
+          grep { $_->[$ARRAY_START_INDICE_PLUS_1] ne $blessed } @expanded;
         push(@{$rc}, {
                       public_methods     => \%public_methods,
                       private_methods    => \%private_methods,
@@ -414,73 +517,48 @@ sub dsread {
 #
 # Internal methods
 #
-sub _concatenateLevels {
-  my ($self, $level) = @_;
-  my @levels = ();
-  while ($level-- > 0) {
-    unshift(@levels, $self->indice_start . ($self->_currentIndicePerLevel->[$level] - 1) . $self->indice_end)
-  }
-  return @levels
-}
-
 sub _pushLevel {
   my ($self, $reftype) = @_;
 
-  push(@{$self->_currentReftypePerLevel}, $reftype);
-  push(@{$self->_currentIndicePerLevel}, $reftype eq 'ARRAY' ? $[ : 0);       # Only used for ARRAY
-  $self->_currentLevel($self->_currentLevel + 1);
-  return
+  push(@{$self->{_currentReftypePerLevel}}, $reftype);
+  push(@{$self->{_currentIndicePerLevel}}, $ARRAY_START_INDICE_MINUS_1); # dsread() will increase it at every item
+  return ++$self->{_currentLevel}
 }
 
 sub _popLevel {
   my ($self) = @_;
 
-  pop(@{$self->_currentReftypePerLevel});
-  pop(@{$self->_currentIndicePerLevel});
-  $self->_currentLevel($self->_currentLevel - 1);
-  return
+  pop(@{$self->{_currentReftypePerLevel}});
+  pop(@{$self->{_currentIndicePerLevel}});
+  return --$self->{_currentLevel}
 }
 
 sub _pushLine {
   my ($self) = @_;
-  push(@{$self->_lines}, ($self->indent x $self->_currentLevel));
-  return
+
+  return push(@{$self->{_lines}}, [ $self->indent x $self->{_currentLevel} ]);
 }
 
 sub _pushDesc {
   my ($self, $what, $desc) = @_;
 
-  if ($what eq 'string') {
+  if ($what eq 'string' && ! looks_like_number($desc)) {
     #
-    # Detect any non ANSI character and enclose within ""
+    # Detect any non ANSI character and enclose result within ""
     #
-    $desc =~ s/$_NON_ASCII_PRINT_RE/sprintf('\\x{%x}', ord(${^MATCH}))/egp;
-    $desc = "\"$desc\"" unless looks_like_number($desc);
+    $desc =~ s/$_NON_ASCII_PRINT_RE/sprintf('\\x{%x}', ord(${^MATCH}))/egpo;
+    $desc = "\"$desc\""
   }
-
   if ($self->with_ansicolor) {
-    my $color = $self->colors->{$what};
-    if (defined($color)) {
-      my $refcolor = reftype($color);
-      if (! $refcolor) {
-        $self->_lines->[-1] .= colored($desc, $color);
-      } elsif ($refcolor eq 'ARRAY') {
-        $self->_lines->[-1] .= colored($color, $desc);
-      } else {
-        $self->_lines->[-1] .= $desc;
-      }
-    } else {
-      $self->_lines->[-1] .= $desc;
-    }
-  } else {
-    $self->_lines->[-1] .= $desc;
+    #
+    # We know that _colors_cache is a HashRef, and that _lines is an ArrayRef
+    #
+    my $color_cache = $self->{_colors_cache}->{$what};  # Handled below if it does not exist or its value is undef
+    $desc = $color_cache . $desc . "\e[0m" if (defined($color_cache))
   }
-  return
-}
+  push(@{$self->{_lines}->[-1]}, $desc);
 
-sub _currentDesc {
-  my ($self) = @_;
-  return $self->_lines->[-1]
+  return
 }
 
 sub _canColor {
@@ -517,7 +595,7 @@ Data::Scan::Impl::Printer - Data::Scan printer implementation
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -605,7 +683,7 @@ Representation of internal indice count end. Default is ']'.
 
 =head2 Bool with_indices_full
 
-Use full internal indice representation, i.e. show indices from the top level up to current level, as if the tree would have been only composed of array references to array references, and so on. Default is a true value.
+Use full internal indice representation, i.e. show indices from the top level up to current level, as if the tree would have been only composed of array references to array references, and so on. Default is a false value.
 
 =head2 Str address_start
 
@@ -645,15 +723,15 @@ Show deparsed subroutine references. Default is a false value.
 
 =head2 Bool with_methods
 
-Show public, private and inherited methods. Default is a true value.
+Show public, private and inherited methods. Default is a false value.
 
 =head2 Bool with_filename
 
 Show loaded or resolved filename. Default is a false value.
 
-=head2 HashRef[Str|ArrayRef] colors
+=head2 HashRef[Str] colors
 
-Explicit ANSI color per functionality. The absence of a color definition means the corresponding value will be printed as-is. A color is defined following the Term::ANSIColor specification, as a string or an array reference.
+Explicit ANSI color per functionality. The absence of a color definition means the corresponding value will be printed as-is. A color is defined following the Term::ANSIColor specification, as a string.
 
 Supported keys of this hash and their eventual default setup is:
 
