@@ -3,11 +3,11 @@ use warnings FATAL => 'all';
 
 package Data::Scan::Printer;
 
-# ABSTRACT: Data::Scan::Printer - Example of a printer consumer for Data::Scan
+# ABSTRACT: Example of a printer consumer for Data::Scan
 
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
-# AUTHORITY
+our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
 use Exporter qw/import/;
 use vars qw/@EXPORT %Option/;
@@ -44,11 +44,11 @@ __END__
 
 =head1 NAME
 
-Data::Scan::Printer - Data::Scan::Printer - Example of a printer consumer for Data::Scan
+Data::Scan::Printer - Example of a printer consumer for Data::Scan
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
