@@ -6,9 +6,9 @@ use Moo::Role;
 
 # ABSTRACT: Data::Scan consumer role
 
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
-# AUTHORITY
+our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
 
 requires 'dsstart';
@@ -32,7 +32,7 @@ Data::Scan::Role::Consumer - Data::Scan consumer role
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 DESCRIPTION
 
